#r "System.Windows.Forms"

using System.Xml;
using System.IO;
using System.Windows.Forms;

//////////////////////////////////////////////////////////////////////////////////////
// 作図部品のインポートスクリプト
// スクリプトウィンドウにコードを入力して、指定した図面ファイルから作図部品を
// インポートすることができます。
//
// 最初に本スクリプトファイルを #load で読み込みます。
//   #load "ComponentIO\Import.csx"
//
// コード入力例 1
//   ImportComponent(@"C:\work\図面1.rpcd", "component1");
//
// コード入力例 2
//   SetDocumentFolder(@"C:\work");
//   ImportComponent("図面1.rpcd", "component1");
//   ImportComponent("図面2.rpcd", "component2");
//
// コード入力例 3 
//   SetFileName(@"c:\work\図面1.rpcd);
//   ImportComponent("component1");
//
// コード入力例 4
//   SetFileName(@"C:\work\図面1.rpcd);
//   ImportComponent(new String[] {"component1", "component2"});
//
//////////////////////////////////////////////////////////////////////////////////////

/// デフォルトのパス名
const String defaultPathName = @"\RootPro CAD 8";

// カレントパスの設定
String defaultPath = System.Environment.GetFolderPath(Environment.SpecialFolder.Personal);
defaultPath = Path.Combine(defaultPath, defaultPathName);

/// <summary>
/// ドキュメントフォルダー
/// </summary>
String documentFolder = defaultPath;

/// <summary>
/// ドキュメントパス
/// </summary>
String documentPath = defaultPath;

/// <summary>
/// ドキュメントフォルダーの設定
/// </summary>
void SetDocumentFolder(String path){documentFolder = path;}

/// <summary>
/// ファイル名の設定
/// </summary>
void SetFileName(String fileName)
{
    // フルパス指定でない場合
    if(!Path.IsPathRooted(fileName))
        fileName = Path.Combine(documentFolder, fileName);

    documentPath = fileName;
}

/// <summary>
/// 作図部品の読み込み
/// </summary>
/// <param name="componentName"></param>
/// <returns></returns>
/// <remarks>
/// 作図部品名を指定して作図部品を現在の図面に読み込む
/// 事前にドキュメントのパス設定が必要
/// </remarks> 
bool ImportComponent(String componentName)
{
    return ImportComponent(documentPath, new String[] {componentName});
}

/// <summary>
/// 作図部品の読み込み
/// </summary>
/// <param name="componentNames"></param>
/// <returns></returns>
/// <remarks>
/// 作図部品名を指定して作図部品を現在の図面に読み込む
/// 事前にドキュメントのパス設定が必要
/// </remarks> 
bool ImportComponent(String[] componentNames)
{
    return ImportComponent(documentPath, componentNames);
}

/// <summary>
/// 作図部品の読み込み
/// </summary>
/// <param name="srcFilePath"></param>
/// <param name="componentName"></param>
/// <returns></returns>
/// <remarks>
/// ファイル名、作図部品名を指定して作図部品を現在の図面に読み込む
/// </remarks> 
bool ImportComponent(String srcFilePath, String componentName)
{
    return ImportComponent(srcFilePath, new String[] {componentName});
}

/// <summary>
/// 作図部品の読み込み
/// </summary>
/// <param name="srcFilePath"></param>
/// <param name="componentNames"></param>
/// <returns></returns>
/// <remarks>
/// ファイル名、作図部品名を指定して作図部品を現在の図面に読み込む
/// </remarks> 
bool ImportComponent(String srcFilePath, String[] componentNames)
{
    // ファイル名を設定
    SetFileName(srcFilePath);

    // ファイル存在チェック
    if(!System.IO.File.Exists(documentPath))
    {
        MessageBox.Show("指定されたファイルが存在しません。\n" + documentPath);
        return false;
    }

    // 現在のドキュメントを取得する
    var distDoc = Application.ActiveDocument;
    if(distDoc == null)
    {
        MessageBox.Show("ファイルが開かれていません。");
        return false;
    }

    if(distDoc.FullName == String.Empty)
    {
        MessageBox.Show("新規図面には読み込むことができません。\n既存の図面を開いてください。");
        return false;
    }

    // 作図部品定義コレクションを取得
    var distMasterDrawComponents = distDoc.MasterDrawComponents;

    // 指定した作図部品がすでに登録されているかチェックする
    foreach(String cName in componentNames)
    {
        if(distMasterDrawComponents[cName] != null)
        {
            MessageBox.Show("指定した作図部品名 [" + cName + "] がすでに登録されています。");
            return false;
        }
    }

    // ドキュメントコレクションを取得する
    var docs = Application.Documents;
    Document srcDoc = null;
    bool bOpenSrcDoc = false;

    try
    {
        int docCount = docs.Count;

        // 指定したファイルが開かれているかチェック
        foreach(Document docTemp in docs)
        {
            if(String.Compare(documentPath, docTemp.FullName, true) == 0)
            {
                srcDoc = docTemp;
                break;
            }
        }
        
        // 開かれていない場合はファイルを開く
        if(srcDoc == null)
        {
            if((srcDoc = docs.Open(documentPath)) == null)
                return false;
            bOpenSrcDoc = true;
        }

        // 作図部品定義コレクションを取得
        var srcMasterDrawComponents = srcDoc.MasterDrawComponents;


        // 指定した作図部品が登録されているかチェックする
        foreach(String cName in componentNames)
        {
            // 作図部品の存在チェック
            if(srcMasterDrawComponents[cName] == null)
            {
                MessageBox.Show("指定した作図部品 [" + cName + "] が定義されていません。");
                continue;
            }

            // 作図部品を追加
            if(AddDrawComponent(distDoc, cName, srcDoc) == null)
            {
                MessageBox.Show("作図部品 [" + cName + "] の追加に失敗しました。");
                return false;
            }
        }
    }
    finally
    {
        // コピー元ドキュメントを閉じる
        if(bOpenSrcDoc == true)
        {
            srcDoc.Close();
            distDoc.Activate();
        }
    }

    return true;
}

/// <summary>
/// 作図部品をドキュメントに追加する
/// </summary>
/// <param name="distDoc">追加先ドキュメント</param>
/// <param name="componentName">作図部品名</param>
/// <param name="srcDoc">コピー元ドキュメント</param>
/// <returns></returns>
/// <remarks>
/// 作図部品をドキュメントに追加する。レイヤが存在しない場合はレイヤも追加する。
/// </remarks> 
MasterDrawComponent AddDrawComponent(Document distDoc, String componentName, Document srcDoc)
{
    // 作図部品を構成する図形を取得する
    var srcShapes = srcDoc.MasterDrawComponents[componentName].Shapes;

    // 作図部品追加メソッド用に図形の配列を用意
    Shape[] shapeArray = new Shape[srcShapes.Count];

    for(int i = 0; i < srcShapes.Count; i++)
    {
        // レイヤ名を取得
        String layerName = srcShapes[i].Layer.Name;
        
        Layer layer = null;
        
        if(layerName != String.Empty)
            // 追加先のドキュメントにレイヤを確保する
            layer = EnsureLayer(distDoc, srcShapes[i].Layer, srcDoc);


        // 作図部品の構成図形が作図部品の場合
        if(srcShapes[i] is DrawComponentShape)
        {
            // 作図部品図形
            var dcs = ((DrawComponentShape)srcShapes[i]);

            // 作図部品名を取得
            var name = dcs.Master.Name;

            // 作図部品定義を取得
            MasterDrawComponent master = distDoc.MasterDrawComponents[name];
            // 追加先に作図部品定義が登録されていない場合は追加する
            if(master == null)
                master = AddDrawComponent(distDoc, name, srcDoc);

            // 作図部品定義IDを指定して作図部品図形を作成する
            shapeArray[i] = ShapeFactory.CreateDrawComponent(master.ID, dcs.Point, dcs.ScaleX, dcs.ScaleY, dcs.Angle);

            // レイヤ(なし)の場合はレイヤIDに 0 を設定
            if(dcs.LayerID == 0)
                shapeArray[i].LayerID = 0;
        }
        else
        {
            // クローンを作成
            shapeArray[i] = srcShapes[i].Clone();
        }

        if(layer != null)
            // レイヤIDを設定
            shapeArray[i].LayerID = layer.ID;
    }

    Point2d origin = Geometry.CreatePoint(0.0, 0.0);
    // ドキュメントに作図部品定義を追加する
    return distDoc.MasterDrawComponents.Add(componentName, shapeArray, origin);
}


/// <summary>
/// ドキュメントにレイヤを追加する
/// </summary>
/// <param name="distDoc">追加先ドキュメント</param>
/// <param name="srcLayer">レイヤ</param>
/// <param name="srcDoc">コピー元ドキュメント</param>
/// <returns></returns>
/// <remarks>
/// ドキュメントにレイヤを追加する。
/// </remarks> 
Layer EnsureLayer(Document distDoc, Layer srcLayer, Document srcDoc)
{
    // 親レイヤ
    Layer parentLayer = null;

    // ルートレイヤ配下の場合
    if(srcLayer.ParetLayer == srcDoc.LayerTable.RootLayer)
        // 親レイヤにルートレイヤを設定
        parentLayer = distDoc.LayerTable.RootLayer;
    else
        // 親レイヤを取得
        parentLayer = EnsureLayer(distDoc, srcLayer.ParetLayer, srcDoc);

    if(parentLayer == null)
        return null;

    Layer ensuredLayer = null;
    // すでにレイヤが存在する場合
    if(parentLayer.ChildLayers[srcLayer.Name] != null)
    {
        ensuredLayer = parentLayer.ChildLayers[srcLayer.Name];
    }
    // レイヤが存在しない
    else
    {
        // レイヤグループ
        if(srcLayer.IsGroup)
        {
            // レイヤグループを追加
            ensuredLayer = parentLayer.ChildLayers.AddLayerGroup(srcLayer.Name);
        }
        // 通常のレイヤ
        else
        {
            // レイヤを追加
            ensuredLayer = parentLayer.ChildLayers.AddLayer(srcLayer.Name);

            // 色、線種、線幅、状態を設定
            ensuredLayer.Color = distDoc.Colors[srcLayer.Color.Number];
            ensuredLayer.State = srcLayer.State;
            ensuredLayer.Hide = srcLayer.Hide;
            if(srcLayer.Linetype.Number < distDoc.Linetypes.Count)
                ensuredLayer.Linetype = distDoc.Linetypes[srcLayer.Linetype.Number];
            ensuredLayer.Linewidth = distDoc.Linewidths[srcLayer.Linewidth.Number];
            ensuredLayer.Printable = srcLayer.Printable;
        }
    }

    return ensuredLayer;
}
